#include <stdio.h>

/*
 *  Convert a PFB file to a PFA file.
 *
 *  A .pfb file is a sequence of packets.  Each packet starts with byte
 *  0x80.  The second byte in the packet gives the type of packet: 1
 *  means it's a packet of ascii data which should be sent out as is
 *  (except for translating \r to the appropriate end-of-line
 *  character(s)); 2 means it's a packet of binary data which should be
 *  translated into hex; 3 means EOF.  For types 1 and 2, the type byte
 *  is followed by four bytes giving the length of the packet, least
 *  significant first.
 *
 * Copyright (c) 1992 by Warren Toomey.		wkt@csadfa.cs.adfa.oz.au
 *
 * You may distribute this code for free (or a nominal copying fee) if you
 * do not alter the source code at all. If you want to distribute a changed
 * version, you should distribute a set of patches along with the original.
 *
 * You may not sell this source code or profit in any way from this code.
 */
#ifdef MSDOS
# define RB "rb"
#else
# define RB "r"
#endif

main(argc, argv)
  int argc;
  char *argv[];
{
  int ch;
  unsigned int count;

  switch(argc)
   {
    case 1: break;
    case 2:
  	if (freopen(argv[1], RB, stdin) == NULL)
    	  { fprintf(stderr, "Could not open %s\n",argv[1]); exit(1); }
	break;
    default: fprintf(stderr,"Usage: pfb2pfa [pfbfile]\n"); exit(1);
   }

  while ((ch = getchar()) != EOF)
  {
    if (ch == 0x80)
    {
      switch (getchar())
      {
	case 1:
	  count = getchar();
	  count += (getchar() << 8);
	  count += (getchar() << 16);
	  count += (getchar() << 24);
	  for (; count; count--)
	  {
	    ch = getchar();
	    if (ch == EOF)
	      exit(1);
	    if (ch == '\r')
	      ch = '\n';
	    putchar(ch);
	  }
	  break;
	case 2:
	  count = getchar();
	  count += (getchar() << 8);
	  count += (getchar() << 16);
	  count += (getchar() << 24);
	  for (; count; count--)
	  {
	    ch = getchar();
	    if (ch == EOF)
	      { fprintf(stderr,"EOF\n"); exit(1); }
	    printf("%02x", ch);
	    if (!(count % 35L))
	      putchar('\n');
	  }
          putchar('\n');
	  break;
	default:
	  exit(0);
      }
    }
    else
      putchar(ch);
  }
}
